// DirectSkeleton Toon Effect - Version 3.0
// Vertex Shader for meshes with FVF buffers and this format:
// D3DFVF_XYZB3|D3DFVF_NORMAL|D3DFVF_DIFFUSE
// Color is taken from vertices
// Silhouette is taken from vSilhAmout or dwSilhAmount
// Silhouette Color Multiplier is taken from Texture (Single Pass)
// Silhouette Color Multiplier is Constant (Dual Pass)

vector vSilhAmount;	// Silhouette Amount (Single Pass)
vector vLightDir;	// Light Direction in View Space

DWORD dwSilhAmount;	// Silhouette Amount (Dual Pass)

matrix mView;		// ViewMatrix
matrix mViewProj;	// View*Proj Matrix
matrix mWorld;		// World Matrix
matrix mWorldB1;	// First Beta World Matrix
matrix mWorldB2;	// Second Beta World Matrix
matrix mWorldB3;	// Third Beta World Matrix

// First Technique, uses a single pass with two textures
technique tecToonSinglePass
{
    pass passAll
    {
        // Load shader constants
        VertexShaderConstant[0] = (0,0.99,0.5,1.0);	// Some useful constants
		VertexShaderConstant[1] = <vLightDir>;		// Light Direction in View Space
		VertexShaderConstant[2] = <vSilhAmount>;	// Silhouette Amount
        VertexShaderConstant[3] = <mView>;			// View Matrix
        VertexShaderConstant[7] = <mViewProj>;		// View*Proj Matrix
        VertexShaderConstant[11] = <mWorld>;		// World Matrix
        VertexShaderConstant[15] = <mWorldB1>;		// Beta1 World Matrix
        VertexShaderConstant[19] = <mWorldB2>;		// Beta2 World Matrix
        VertexShaderConstant[23] = <mWorldB3>;		// Beta3 World Matrix
        VertexShaderConstant[27] = (-0.026,0.085,0.0,0.0);	// More useful constants

		AlphaTestEnable = FALSE;
		AlphaBlendEnable = FALSE;

		ColorOp[0]   = Modulate;
        ColorArg1[0] = Texture;
        ColorArg2[0] = Diffuse;
        AlphaOp[0]   = SelectArg1;
        AlphaArg1[0] = Diffuse;

        ColorOp[1]   = Modulate;
        ColorArg1[1] = Texture;
        ColorArg2[1] = Current;
        AlphaOp[1]   = SelectArg1;
        AlphaArg1[1] = Current;

        ColorOp[2]   = Disable;
        AlphaOp[2]   = Disable;

        // Definition of the vertex shader, declarations then assembly.
        VertexShader =
        decl
        {
            stream 0;

			// We use FVF for out meshes, so use FVF compatible registers
            float v0[3];	// Position
            float v1[3];	// Beta Weights
            float v3[3];	// Normal
            d3dcolor v5[1];	// Diffuse
        }
        asm
        {
			vs.1.0					// Version instruction

			// Calculate the last weight (save in r0)
			add		r0.x, c0.w, -v1.x	// 1.0 - Beta1 - Beta2 - Beta3
			add		r0.x, r0.x, -v1.y
			add		r0.x, r0.x, -v1.z

			mov		r5.w, c0.w
			mov		r6.w, c0.w
			mov		r2.w, c0.w
			mov		r3.w, c0.w

			// Skin with the first matrix
			//m4x3	r5, v0, c11
			dp4		r5.x, v0, c11			
			dp4		r5.y, v0, c12
			dp4		r5.z, v0, c13
			
			//m3x3	r6, v3, c11
			dp3		r6.x, v3, c11
			dp3		r6.y, v3, c12
			dp3		r6.z, v3, c13

			// Blend it
			mul		r5, r5, v1.xxxx
			mul		r6, r6, v1.xxxx

			// Skin with the second matrix
			//m4x3	r2, v0, c15
			dp4		r2.x, v0, c15
			dp4		r2.y, v0, c16
			dp4		r2.z, v0, c17
			
			//m3x3	r3, v3, c15
			dp3		r3.x, v3, c15
			dp3		r3.y, v3, c16
			dp3		r3.z, v3, c17

			// Blend it
			mad		r5, r2, v1.yyyy, r5
			mad		r6, r3, v1.yyyy, r6

			// Skin with the third matrix
			//m4x3	r2, v0, c19
			dp4		r2.x, v0, c19
			dp4		r2.y, v0, c20
			dp4		r2.z, v0, c21
			
			//m3x3	r3, v3, c19
			dp3		r3.x, v3, c19
			dp3		r3.y, v3, c20
			dp3		r3.z, v3, c21

			// Blend it
			mad		r5, r2, v1.zzzz, r5
			mad		r6, r3, v1.zzzz, r6

			// Skin with the fourth matrix
			//m4x3	r2, v0, c23
			dp4		r2.x, v0, c23
			dp4		r2.y, v0, c24
			dp4		r2.z, v0, c25

			//m3x3	r3, v3, c23
			dp3		r3.x, v3, c23
			dp3		r3.y, v3, c24
			dp3		r3.z, v3, c25

			// Blend it (use the final beta weight)
			mad		r5, r2, r0.xxxx, r5
			mad		r6, r3, r0.xxxx, r6

			// Compute Position
			mov		r5.w, c0.w		// r5 = World space position
			//m4x4	oPos, r5, c7	// Transform vertices by view/projection matrix
			dp4		oPos.x, r5, c7
			dp4		oPos.y, r5, c8
			dp4		oPos.z, r5, c9
			dp4		oPos.w, r5, c10

			mov		oD0, v5		// Output the Diffuse color

			// Normalize Normal
			dp3		r1.x, r6.xyz, r6.xyz	// Load the square into r1
			rsq		r1.xyz, r1.x			// Get the inverse of the square
			mul		r6.xyz, r6.xyz, r1.xyz	// Multiply (r6 = World space normal)

			// Color Banding
			//m3x3	r6, r6, c3		// r6 = View space normal
			dp3		r6.x, r6, c3
			dp3		r6.y, r6, c4
			dp3		r6.z, r6, c5
			
			dp3		r1, -r6, c1		// r1 = Diffuse lighting calc
			add		r1, r1, c27.y	// Add Patch
			max     r1, r1, c0.x	// Clamp r1 to zero
			min     r1, r1, c0.y	// Clamp r1 to .99
			mov		oT0.x, r1		// Emit tex-coord 0, (r1,0.0)
			//add		oT0.x, r1.x, c27.y	// Emit tex-coord 0, (r1,0.0)
			mov     oT0.y, c0.x

			// View space position
			//m4x4	r5, r5, c3		// r5 = View space position
			dp4		r5.x, r5, c3
			dp4		r5.y, r5, c4
			dp4		r5.z, r5, c5
			//dp4		r5.w, r5, c6

			// Normalize view space position
			dp3		r1.x, r5.xyz, r5.xyz	// Load the square into r1
			rsq		r1.xyz, r1.x			// Get the inverse of the square
			mul		r5.xyz, r5.xyz, r1.xyz	// Multiply (r5 = Normalized Position)

			// Silhouette Calculation
			dp3		r1.x, r5, r6			// Compute r1 = Position.Normal
			mad		r1.x, r1.x, c0.z, c0.z	// Scale to [0,1]
			add		r1.x, r1.x, c2.x		// Add Silhouette Modifier
			add		r1.x, r1.x, c27.x		// Add patch
			max     r1.x, r1.x, c0.x		// Clamp r1 to zero
			min     oT1.x, r1.x, c0.y		// Clamp r1.x to .99 and emit as tex-coord 1
			mov		oT1.y, c0.x				// Emit oT1 (r1.x,0.0)
        };
    }
}

// Second Technique, uses two passes with one texture
technique tecToonDualPass
{
    pass passColor
    {
        // Load shader constants
        VertexShaderConstant[0] = (0,0.99,0.5,1.0);	// Some useful constants
		VertexShaderConstant[1] = <vLightDir>;		// Light Direction in View Space
        VertexShaderConstant[2] = <mView>;			// View Matrix
        VertexShaderConstant[6] = <mViewProj>;		// View*Proj Matrix
        VertexShaderConstant[10] = <mWorld>;		// World Matrix
        VertexShaderConstant[14] = <mWorldB1>;		// Beta1 World Matrix
        VertexShaderConstant[18] = <mWorldB2>;		// Beta2 World Matrix
        VertexShaderConstant[22] = <mWorldB3>;		// Beta3 World Matrix

		AlphaTestEnable = FALSE;
		AlphaBlendEnable = FALSE;

		ColorOp[0]   = Modulate;
        ColorArg1[0] = Texture;
        ColorArg2[0] = Diffuse;
        AlphaOp[0]   = SelectArg1;
        AlphaArg1[0] = Diffuse;

        ColorOp[1]   = Disable;
        AlphaOp[1]   = Disable;

        // Definition of the vertex shader, declarations then assembly.
        VertexShader =
        decl
        {
            stream 0;

			// We use FVF for out meshes, so use FVF compatible registers
            float v0[3];	// Position
            float v1[3];	// Beta Weights
            float v3[3];	// Normal
            d3dcolor v5[1];	// Diffuse
        }
        asm
        {
			vs.1.0					// Version instruction

			// Calculate the last weight (save in r0)
			add		r0.x, c0.w, -v1.x	// 1.0 - Beta1 - Beta2 - Beta3
			add		r0.x, r0.x, -v1.y
			add		r0.x, r0.x, -v1.z

			mov		r5.w, c0.w
			mov		r6.w, c0.w
			mov		r2.w, c0.w
			mov		r3.w, c0.w

			// Skin with the first matrix
			//m4x3	r5, v0, c10
			dp4		r5.x, v0, c10
			dp4		r5.y, v0, c11
			dp4		r5.z, v0, c12

			//m3x3	r6, v3, c10
			dp3		r6.x, v3, c10
			dp3		r6.y, v3, c11
			dp3		r6.z, v3, c12

			// Blend it
			mul		r5, r5, v1.xxxx
			mul		r6, r6, v1.xxxx

			// Skin with the second matrix
			//m4x3	r2, v0, c14
			dp4		r2.x, v0, c14
			dp4		r2.y, v0, c15
			dp4		r2.z, v0, c16

			//m3x3	r3, v3, c14
			dp3		r3.x, v3, c14
			dp3		r3.y, v3, c15
			dp3		r3.z, v3, c16

			// Blend it
			mad		r5, r2, v1.yyyy, r5
			mad		r6, r3, v1.yyyy, r6

			// Skin with the third matrix
			//m4x3	r2, v0, c18
			dp4		r2.x, v0, c18
			dp4		r2.y, v0, c19
			dp4		r2.z, v0, c20

			//m3x3	r3, v3, c18
			dp3		r3.x, v3, c18
			dp3		r3.y, v3, c19
			dp3		r3.z, v3, c20

			// Blend it
			mad		r5, r2, v1.zzzz, r5
			mad		r6, r3, v1.zzzz, r6

			// Skin with the fourth matrix
			//m4x3	r2, v0, c22
			dp4		r2.x, v0, c22
			dp4		r2.y, v0, c23
			dp4		r2.z, v0, c24

			//m3x3	r3, v3, c22
			dp3		r3.x, v3, c22
			dp3		r3.y, v3, c23
			dp3		r3.z, v3, c24

			// Blend it (use the final beta weight)
			mad		r5, r2, r0.xxxx, r5
			mad		r6, r3, r0.xxxx, r6

			// Compute Position
			mov		r5.w, c0.w		// r5 = World space position
			//m4x4	oPos, r5, c6	// Transform vertices by view/projection matrix
			dp4		oPos.x, r5, c6
			dp4		oPos.y, r5, c7
			dp4		oPos.z, r5, c8
			dp4		oPos.w, r5, c9

			mov		oD0, v5		// Output the Diffuse color

			// Normalize Normal
			dp3		r1.x, r6.xyz, r6.xyz	// Load the square into r1
			rsq		r1.xyz, r1.x			// Get the inverse of the square
			mul		r6.xyz, r6.xyz, r1.xyz	// Multiply (r6 = World space normal)

			// Color Banding
			//m3x3	r0, r6, c2		// r0 = View space normal
			dp3		r0.x, r6, c2
			dp3		r0.y, r6, c3
			dp3		r0.z, r6, c4

			dp3		r1, -r0, c1		// r1 = Diffuse lighting calc
			max     r1, r1, c0.x	// Clamp r1 to zero
			min     r1, r1, c0.y	// Clamp r1 to .99
			mov		oT0.x, r1		// Emit tex-coord 0, (r1,0.0)
			mov     oT0.y, c0.x
        };
    }

    pass passSilh
    {
        // Load shader constants
        VertexShaderConstant[0] = (-0.05,0.5,1.0,0.3);	// Some useful constants
        //VertexShaderConstant[1] = <vLightDir>;			// Light Direction in View Space
        //VertexShaderConstant[2] = <mView>;		// View Matrix
        //VertexShaderConstant[6] = <mViewProj>;	// View*Proj Matrix
        //VertexShaderConstant[10] = <mWorld>;	// World Matrix
        //VertexShaderConstant[14] = <mWorldB1>;	// Beta1 World Matrix
        //VertexShaderConstant[18] = <mWorldB2>;	// Beta2 World Matrix
        //VertexShaderConstant[22] = <mWorldB3>;	// Beta3 World Matrix

		AlphaTestEnable = TRUE;
		AlphaBlendEnable = FALSE;
		AlphaFunc = GreaterEqual;
		AlphaRef = <dwSilhAmount>;	// Silhouette Amount

		ColorOp[0]   = SelectArg1;
        ColorArg1[0] = Diffuse;
        AlphaOp[0]   = SelectArg1;
        AlphaArg1[0] = Diffuse;

        ColorOp[1]   = Disable;
        AlphaOp[1]   = Disable;

        // Definition of the vertex shader, declarations then assembly.
        VertexShader =
        decl
        {
            stream 0;

			// We use FVF for out meshes, so use FVF compatible registers
            float v0[3];	// Position
            float v1[3];	// Beta Weights
            float v3[3];	// Normal
            d3dcolor v5[1];	// Diffuse
        }
        asm
        {
			vs.1.0							// Version instruction

			// Calculate the last weight (save in r0)
			add		r0.x, c0.z, -v1.x	// 1.0 - Beta1 - Beta2 - Beta3
			add		r0.x, r0.x, -v1.y
			add		r0.x, r0.x, -v1.z

			mov		r5.w, c0.w
			mov		r6.w, c0.w
			mov		r2.w, c0.w
			mov		r3.w, c0.w

			// Skin with the first matrix
			//m4x3	r5, v0, c10
			dp4		r5.x, v0, c10
			dp4		r5.y, v0, c11
			dp4		r5.z, v0, c12

			//m3x3	r6, v3, c10
			dp3		r6.x, v3, c10
			dp3		r6.y, v3, c11
			dp3		r6.z, v3, c12

			// Blend it
			mul		r5, r5, v1.xxxx
			mul		r6, r6, v1.xxxx

			// Skin with the second matrix
			//m4x3	r2, v0, c14
			dp4		r2.x, v0, c14
			dp4		r2.y, v0, c15
			dp4		r2.z, v0, c16

			//m3x3	r3, v3, c14
			dp3		r3.x, v3, c14
			dp3		r3.y, v3, c15
			dp3		r3.z, v3, c16

			// Blend it
			mad		r5, r2, v1.yyyy, r5
			mad		r6, r3, v1.yyyy, r6

			// Skin with the third matrix
			//m4x3	r2, v0, c18
			dp4		r2.x, v0, c18
			dp4		r2.y, v0, c19
			dp4		r2.z, v0, c20

			//m3x3	r3, v3, c18
			dp3		r3.x, v3, c18
			dp3		r3.y, v3, c19
			dp3		r3.z, v3, c20

			// Blend it
			mad		r5, r2, v1.zzzz, r5
			mad		r6, r3, v1.zzzz, r6

			// Skin with the fourth matrix
			//m4x3	r2, v0, c22
			dp4		r2.x, v0, c22
			dp4		r2.y, v0, c23
			dp4		r2.z, v0, c24

			//m3x3	r3, v3, c22
			dp3		r3.x, v3, c22
			dp3		r3.y, v3, c23
			dp3		r3.z, v3, c24

			// Blend it (use the final beta weight)
			mad		r5, r2, r0.xxxx, r5
			mad		r6, r3, r0.xxxx, r6

			// Compute Position
			mov		r5.w, c0.z		// r5 = World space position
			//m4x4	oPos, r5, c6	// Transform vertices by view/projection matrix
			dp4		oPos.x, r5, c6
			dp4		oPos.y, r5, c7
			dp4		oPos.z, r5, c8
			dp4		oPos.w, r5, c9

			// View space position
			//m4x4	r5, r5, c2		// r5 = View space position
			dp4		r5.x, r5, c2
			dp4		r5.y, r5, c3
			dp4		r5.z, r5, c4
			//dp4		r5.w, r5, c5

			// Normalize view space position
			dp3     r1.x, r5.xyz, r5.xyz
			rsq     r1.xyz, r1.x
			mul     r5.xyz, r5.xyz, r1.xyz	// r5 = Normalized Position

			// Normalize Normal
			dp3		r1.x, r6.xyz, r6.xyz	// Load the square into r1
			rsq		r1.xyz, r1.x			// Get the inverse of the square
			mul		r6.xyz, r6.xyz, r1.xyz	// Multiply

			// Silhouette Calculation
			//m3x3	r6, r6, c2				// View space normal
			dp3		r6.x, r6, c2
			dp3		r6.y, r6, c3
			dp3		r6.z, r6, c4

			dp3		r1.x, r5, r6			// Compute dot product (vViewDir.vNormal)
			add		r1.x, r1.x, c0.x		// Add patch
			mad		oD0.w, r1.x, c0.y, c0.y	// Scale to [0,1] and emit as diffuse alpha
			mul		oD0.xyz, c0.w, v5		// Emit black diffuse color component
        };
    }
}